/*
 * Firmware_Multimeter.c
 *
 * Created: 30.04.2020
 * Author : Topsoft
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <util/delay.h>

#define RXD0						PD0							// Bluetooth TXD 38400,8,N,1
#define TXD0						PD1							// Bluetooth RXD / not use
#define CTS0						PD2							// Bluetooth CTS / not use
#define RTS0						PD3							// Bluetooth RTS / not use
#define BTR							PD4							// Bluetooth Reset

#define TXD1						PB3							// not use and connect
#define RXD1						PB4							// Multimeter 2400,8,N,1

static uint8_t buf_index = 0;									// Zeiger auf aktuellen Buffereintrag
static uint8_t buf_sync = 0;									// Flag fr Startbytes
static volatile uint8_t buf_send = 0;							// Flag fr Frame komplet
static volatile uint8_t buf[10];								// der Buffer

void bluetooth_init(void){										// Reset des Bluetooth Modul
	DDRD |= (1<<BTR);											// Ausgang / Reset Bluetooth
	PORTD |= (1<<BTR);											// High
	_delay_ms(10);												// warte
	PORTD &= ~(1<<BTR);											// Low
}

void usart0_init(void){											// sendet per Bluetooth
	#define BAUD 38400											// unter 9600 Baud senden meine Bluetooth Module nur Mll !!!!!!!!!!
	#include <util/setbaud.h>
	UBRR0 = UBRR_VALUE;
	#if USE_2X
	UCSR0A |= (1 << U2X0);
	#else
	UCSR0A &= ~(1 << U2X0);
	#endif
	UCSR0B |= (1<<TXEN0);										// UART0 TX einschalten
	UCSR0C = (1<<UCSZ01)|(1<<UCSZ00);							// Asynchron 8N1
	#undef BAUD
}

void usart1_init(void){											// empfngt vom Multimeter
	#define BAUD 2400
	#include <util/setbaud.h>
	UBRR1 = UBRR_VALUE;
	#if USE_2X
	UCSR1A |= (1 << U2X1);
	#else
	UCSR1A &= ~(1 << U2X1);
	#endif
	UCSR1B |= (1<<RXEN1)|(1<<RXCIE1);							// UART1 RX und Interrupt einschalten
	UCSR1C = (1<<UCSZ11)|(1<<UCSZ10);							// Asynchron 8N1
	#undef BAUD
}

ISR(USART1_RX_vect){											// Byte vom Multimeter empfangen
	//PORTC |= (1<<PC0);
	uint8_t byte = UDR1;										// einlesen
	if(!buf_sync){												// Startbytes noch nicht empfangen
		if((byte == 0xDC) && (buf_index == 0)){					// erste Startbyte
			buf_index++;
			buf[0] = byte;
		}else if((byte == 0xBA) && (buf_index == 1)){			// zweite Startbyte
			buf_index++;
			buf[1] = byte;
			buf_sync = 1;
		}
	}else{														// Startbytes schon empfangen
		buf[buf_index++] = byte;								// Byte in Buffer
		if (buf_index > 9){										// Frame vollstndig empfangen
				buf_index = 0;
				buf_sync = 0;
				buf_send = 1;									// main kann Frame senden
		}
	}
	//PORTC &= ~(1<<PC0);
}

// der Programmablauf funktioniert so nur weil
// 1. es wird schneller gesendet als empfangen
// 2. das senden eines Frame schneller ist als der Abstand zwischen zwei Frame

int main(void){
	usart0_init();
	bluetooth_init();
	usart1_init();
	//DDRC |= (1<<PC0)|(1<<PC1)|(1<<PC2)|(1<<PC3)|(1<<PC4)|(1<<PC5);
	//PORTC &= ~((1<<PC0)|(1<<PC1)|(1<<PC2)|(1<<PC3)|(1<<PC4)|(1<<PC5));
	sei();
	for(;;){
		if (buf_send){
			//PORTC |= (1<<PC1);
			for(uint8_t i = 0; i < 10; i++){					// 1 Frame = 10 Byte
				while ( !( UCSR0A & (1<<UDRE0)));				// warte bis Transmit Buffer leer
				UDR0 = buf[i];									// senden
			}
			buf_send = 0;
		}
		set_sleep_mode(SLEEP_MODE_IDLE);
		sleep_mode();
		//PORTC &= ~(1<<PC1);
	}
	return 0;
}

